#!/bin/sh

# Enable Modem
. /etc/ssvconfig/bin/ssvhardware
hwr_modem on

# Create device link
[ -L /dev/modem2 ] || ln -s /dev/ttyUSB2 /dev/modem2

# Run daemon as Python script
sleep 5
exec /usr/bin/python <<PYTHON
#!/usr/bin/env python

"""

 File: smsgateway.py

 Installation:
 - Install Python 2.7, that is included in gApp 'python3', version 3.5.3
   Menu: System, Apps management
   View online apps
   Locate 'python3' and click install

 - Open command line via Telnet, SSH or ShellInTheBox,
   and Install module 'serial' for Python 2.7
 
   remount rw
   apt-get update
   apt-get install python-serial
   remount ro
 
 - Save this script as 'autostart'
   Menu: Services, Autostart

 - Persist the settings
   Menu: System management

 - Disable the firewall, or allow port 6000/TCP as incomming port
 
 - Check syslog for this line:
   SMS Server waiting on port 6000

 Usage:
 - Connect via TCP/IP port 6000 to the device and
   send a text line starte with telefonnumer, space and
   end the text with CRLF, for example
   +491234567890 Message CR LF
 - Use + Country Number as mobile number!

 Debugging:
 - Disable the autostart in Menu!

 - Start service on command line
   cd /media/data
   ./smsgateway.py 6000 /dev/modem2

 - Use an other PC and connect via telnet to RMG/941 TCP-port 6000

 - You should see the message

   SMSgateway v1.0

 - Type the destination number and a text in single line and enter with CRLF:

   +491234567890 Test message

 - After message was send the answer in telnet would be:

   OK: SMS accepted

 - Test with netCat:

   echo -ne '01234567890 A message for SMS\r\n' | nc -w 60 192.168.0.126 6000

 - As an answer should see:

   SMSgateway v1.0
   OK: SMS accepted
 
 - See into syslog for informations

 - Edit the script and set some oth the DEBUG_RX... to True
   for more details about serial communication.
"""

import SocketServer
import serial
import re
from sys import argv, exit
from syslog import openlog, syslog, LOG_PERROR, LOG_ERR, LOG_INFO, LOG_DEBUG
from threading import Thread
from time import sleep, time

ENABLE_DCD_CHECK = True

DEBUG_RX_BYTE = False
DEBUG_RXTX_LINE = False
DEBUG_SMS_SIMULATION = False
timestamp_SMS = {}

# Default listen port 6000
if len(argv) > 1:
    global_port = int(argv[1])
else:
    global_port = 6000

# Modem-Port als Parameter
if len(argv) > 2:
    SERIAL_PORT = argv[2]
else:
    SERIAL_PORT = "/dev/modem2"

openlog("smsgateway", LOG_PERROR, LOG_INFO)


def buffered_readlines(pull_next_chunk, send_chunk=None, buf_size=4096, chunk_char='\r'):
  """
  https://stackoverflow.com/questions/822001/python-sockets-buffering (6)
    pull_next_chunk is callable that should accept one positional argument max_len,
    i.e. socket.recv or file().read and returns string of up to max_len long or
  empty one when nothing left to read.

  ...
  >>> # the following code won't read whole file into memory
  ... # before splitting it into lines like .readlines method
  ... # of file does. Also it won't block until FIFO-file is closed
  ...
  >>> for line in buffered_readlines(open('huge_file').read):
  ...   # process it on per-line basis
  ...
  >>>
  """
  chunks = []
  while True:
    chunk = pull_next_chunk(buf_size)

    if not chunk:
      if chunks:
        yield ''.join(chunks)
      break

    # NL beim Testen entfernen
    if DEBUG_SMS_SIMULATION:
      if chunk_char == '\r':
        chunk = chunk.replace('\n', '')

    if DEBUG_RX_BYTE:
      dbg = chunk.replace('\r', '[CR]')
      dbg = dbg.replace('\n', '[NL]')
      syslog(LOG_DEBUG, 'rx "' + dbg + '" len:' + str(len(chunk)))

    if not chunk_char in chunk:
      chunks.append(chunk)
      # echo back (TCP)
      if send_chunk != None:
        send_chunk(chunk)
      continue

    chunk = chunk.split(chunk_char)
    if chunks:
      yield ''.join(chunks + [chunk[0]])
    else:
      yield chunk[0]
    for line in chunk[1:-1]:
      yield line
    if chunk[-1]:
      chunks = [chunk[-1]]
    else:
      chunks = []

def modem_waitOK(ser, response='OK'):
    for data in buffered_readlines(ser.read, None, 1, '\n'):
        antwort = data.strip('\r')
        anzahl = len(antwort)
        if DEBUG_RXTX_LINE and anzahl != 0:
            syslog(LOG_DEBUG, 'RX: "' + antwort + '" len:' + str(anzahl))
        if antwort == 'ERROR' or antwort == 'NO CARRIER':
            syslog(LOG_ERR, 'Modem Failed with ' + antwort)
            return False
        elif antwort == response:
            return True

    # Clear buf
    #sleep(1)
    ser.read(ser.inWaiting())

    syslog(LOG_DEBUG, 'modem_waitOK: Failed with timeout')
    return False

def modem_command(ser, cmd, response='OK'):

    # Clear buf
    sleep(0.2)
    ser.read(ser.inWaiting())

    # Send Command
    if DEBUG_RXTX_LINE:
        syslog(LOG_INFO, 'TX: ' + cmd)
    ser.write(cmd) # Send Command

    # Wait for OK
    if modem_waitOK(ser, response):
        return True

    return False

def modem_sendSMS(ser, funknummer, nachricht):
    # Disable echo
    if not modem_command(ser, "ATV1E0\r"):
        return False

    # Command supported?
    #modem_command(ser, 'AT+CMGD=?\r')

    # List all messages
    #modem_command(ser, 'AT+CMGL="ALL"\r')

    # delete all Send SMS
    # 3. Meaning: Ignore the value of index and delete all SMS messages whose status is "received read", "stored unsent" or "stored sent" from the message storage area.
    modem_command(ser, 'AT+CMGD=0,3\r')

    # set to text mode
    if not modem_command(ser, "AT+CMGF=1\r"):
        return False

    if DEBUG_SMS_SIMULATION:
        syslog(LOG_DEBUG, 'SMS-Simulation an:' + funknummer + ' Text:' + nachricht)
        sleep(2)
        return True

    # Send Command for SMS
    cmd = 'AT+CMGS="' + funknummer + '"\r'
    if DEBUG_RXTX_LINE:
        syslog(LOG_INFO, 'TX: ' + cmd)
    ser.write(cmd)

    # Wait for Prompt
    if modem_waitOK(ser, '> '):
        # Send Message
        if DEBUG_RXTX_LINE:
            syslog(LOG_INFO, 'TX: ' + nachricht + '[26]')
        ser.write(nachricht + chr(26))
        # Wait for OK
        if modem_waitOK(ser):
            return True
    return False

def check_last_SMS(funknummer):
    if timestamp_SMS.has_key(funknummer):

        last = timestamp_SMS[funknummer]
        current = time()
        elapsed = current - last

        # Letzte SMS aelter als 5 Minuten?
        if elapsed < 5*60:
            # Nicht senden
            syslog(LOG_INFO, "Letzte SMS war vor %s Sekunden. SMS wird ignoriert." % elapsed)
            return True

    # Nicht im Cache, also SMS senden
    return False

def sendSMS(funknummer, nachricht):

    if check_last_SMS(funknummer):
        sleep(1)
        return True

    try:

        # Open serial port (SMS needs 7 seconds to send)
        ser = serial.Serial(SERIAL_PORT, baudrate = 9600, timeout = 15)
        if not ser:
            return False

    except Exception as e:

        syslog(LOG_INFO, "SERIAL open Exception: %s" % e.args[0])
        return False

    ret = modem_sendSMS(ser, funknummer, nachricht)
    ser.close()

    if ret:
        syslog(LOG_INFO, "SMS Message send to " + funknummer)
        # Zeitstempel fuer diese Nummer ablegen
        timestamp_SMS[funknummer] = time()

    return ret

class SmsTCPSocketHandler(SocketServer.BaseRequestHandler):
    """
    The RequestHandler class for our server.

    It is instantiated once per connection to the server, and must
    override the handle() method to implement communication to the
    client.
    """

    def handle(self):
        # self.request is the TCP socket connected to the client
        self.request.settimeout(60)
        syslog(LOG_INFO, "Connection from " + str(self.client_address[0]))

        # Start message to clinet
        self.request.sendall(
            'SMSgateway v1.0\r\n')

        prog = re.compile('([+0][0-9]{5,30}) (.{1,160})')
        try:
            for self.data in buffered_readlines(self.request.recv, self.request.sendall):
                syslog(LOG_INFO, 'Client wrote: ' + format(self.data) + ' len:' + str(len(self.data)))
                match = prog.match(self.data)
                if match:
                    funknummer = match.group(1)
                    nachricht = match.group(2)

                    # Deutsche Vorwahl?
                    start_nr = funknummer[0:1]
                    if start_nr == '0':
                        funknummer = '+49' + funknummer[1:]

                    break
                else:
                    syslog(LOG_DEBUG, 'Not match:' + self.data)
        except:
            syslog(LOG_ERR, "Error: Timeout on input")
            self.request.sendall('\r\nTimeout.\r\n')
            return

        syslog(LOG_INFO, "Message accepted with " + str(len(nachricht)) + " bytes for " + funknummer)

        if sendSMS(funknummer, nachricht):
            print('Send OK')
            self.request.sendall('OK: SMS accepted\r\n')
            # Warte ein wenig, um das NO CARRIER zu verzoegern
            sleep(1)
        else:
            syslog(LOG_ERR, "SMS Message not send to " + funknummer)
            print('Failed')
            self.request.sendall('ERROR: SMS failed\r\n')

if __name__ == "__main__":

    # Als Test direkt von der Kommandozeile eine SMS senden
    if len(argv) > 4:
        sendSMS(argv[3], argv[4])
        print('Program exit now')
        exit()

    # For localhost only
    #HOST, PORT = "localhost", global_port
    # For all from internet
    HOST, PORT = "0.0.0.0", global_port

    # instantiate the server, and bind to localhost on port 6000
    server = SocketServer.TCPServer((HOST, PORT), SmsTCPSocketHandler)
    syslog(LOG_INFO, "SMS Server waiting on port "+str(PORT))

    # activate the server
    # this will keep running until Ctrl-C
    server.serve_forever()
PYTHON
